﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Activity : MonoBehaviour {	

	public enum ActivityType
	{
		MOVING_FORWARD,
		MOVING_BACKWARD,
		BUSY,
		WAITING
	}

	// Use this for initialization
	void Awake () {
		startedRunning = false;
		running = false;

		path = new List<Vector2> ();
		path.Add (new Vector2 (1, 1));
		path.Add (new Vector2 (1, 3));
		path.Add (new Vector2 (-3, 2));
		nextPathNodeIndex = 0;
		activityType = ActivityType.WAITING;

		mOwner = null;
		Debug.Log ("activity's room (awake): " + currentRoom.ToString ());
	}	

	void Start()
	{
		((Character)gameObject.GetComponent("Character")).isActive = false;		
		Debug.Log ("activity's room (start): " + currentRoom.ToString ());
		Debug.Log ("activity's title: " + title);

		isWife = gameObject.name == "wife" ? true : false;
	}

	public void Run(){
		Character tCharacter = gameObject.GetComponent<Character> ();
		mOwner = gameObject;
		elapsedTime = 0;
		activityType = ActivityType.WAITING;
		startingPositionAhead = false;

		Debug.Log ("activity's room (run): " + currentRoom.ToString ());
		if (tCharacter.currentRoom == currentRoom) {
			changingRoom = false;
			if(choosenPath != null)
			{
				choosenPath.Clear();
			}
			Debug.Log("name: " + gameObject.name);
			choosenPath = new List<Vector2>(path);
//			choosenPath = path;
			if ((currentRoom == Character.CurrentRoom.LEFT) && isWife) {
				choosenPath.Insert (0, Character.leftPositionWife);
			} else if ((currentRoom == Character.CurrentRoom.LEFT) && !isWife) {
				choosenPath.Insert (0, Character.leftPositionHusband);
			} else if ((currentRoom == Character.CurrentRoom.RIGHT) && isWife) {
				choosenPath.Insert (0, Character.rightPositionWife);
			} else if ((currentRoom == Character.CurrentRoom.RIGHT) && !isWife) {
				choosenPath.Insert (0, Character.rightPositionHusband);
			}

			++nextPathNodeIndex;
		} else {
			changingRoom = true;
			if(choosenPath != null)
			{
				choosenPath.Clear();
			}
			choosenPath = new List<Vector2>(otherPath);
//			choosenPath = otherPath;
		}

		startedRunning = true;
		running = false;

		Debug.Log ("changing the roon: " + changingRoom);
		Debug.Log ("choosen room count: " + choosenPath.Count);

		tCharacter.currentRoom = currentRoom;
	}

	// Update is called once per frame
	void FixedUpdate () {
		if ((activityType == ActivityType.BUSY) && !startedRunning && !running) 
		{
			elapsedTime += Time.fixedDeltaTime;
			if(elapsedTime >= duration)
			{
				startedRunning = true;
				elapsedTime = 0;
				Debug.Log ("wyruszam przed: " + path.Count);
				if(choosenPath != null)
				{
					choosenPath.Clear();
				}
				choosenPath = new List<Vector2>(path);
				Debug.Log ("wyruszam po: " + path.Count);
//				choosenPath = path;

				if ((currentRoom == Character.CurrentRoom.LEFT) && isWife) {
					choosenPath.Insert (0, Character.leftPositionWife);
				} else if ((currentRoom == Character.CurrentRoom.LEFT) && !isWife) {
					choosenPath.Insert (0, Character.leftPositionHusband);
				} else if ((currentRoom == Character.CurrentRoom.RIGHT) && isWife) {

					choosenPath.Insert (0, Character.rightPositionWife);
				} else if ((currentRoom == Character.CurrentRoom.RIGHT) && !isWife) {
					choosenPath.Insert (0, Character.rightPositionHusband);
				}
				Debug.Log ("wyruszam po: " + choosenPath.Count);
				nextPathNodeIndex = choosenPath.Count - 2;
				Debug.Log ("wyruszam nextPathNodeIndex: " + nextPathNodeIndex);
			}
		}

		if(startedRunning && ((activityType == ActivityType.BUSY) || (activityType == ActivityType.WAITING)))
		{
			Debug.Log ("nextPathNodeIndex: " + nextPathNodeIndex);
			Debug.Log ("choosenPath.Count: " + choosenPath.Count);
			currentDirection = (choosenPath[nextPathNodeIndex] - new Vector2(transform.position.x, transform.position.y)).normalized;
			startedRunning = false;
			running = true;		

			if (activityType == ActivityType.WAITING) {
				activityType = ActivityType.MOVING_FORWARD;				
			}
			
			if (activityType == ActivityType.BUSY) {
				activityType = ActivityType.MOVING_BACKWARD;
				((Character)gameObject.GetComponent("Character")).isActive = false;		
			}
		}

		if (running && (Vector2.Distance(choosenPath[nextPathNodeIndex], this.transform.position) < 0.01)) {
			bool tGoingToBeBusy = nextPathNodeIndex >= (choosenPath.Count - 1);
			bool tGoingToWait = nextPathNodeIndex <= 0;

			if(tGoingToBeBusy && (activityType == ActivityType.MOVING_FORWARD))
			{
				activityType = ActivityType.BUSY;
				startedRunning = false;
				running = false;
				((Character)gameObject.GetComponent("Character")).isActive = true;		
			}
			else if(tGoingToWait && (activityType == ActivityType.MOVING_BACKWARD))
			{
				activityType = ActivityType.WAITING;
				startedRunning = false;
				running = false;
				Destroy(this);
			}
			else
			{
				if(activityType == ActivityType.MOVING_FORWARD)
				{
					++nextPathNodeIndex;
				}
				else if(activityType == ActivityType.MOVING_BACKWARD)
				{
					--nextPathNodeIndex;
//					++nextPathNodeIndex;
				}			
			}
		}
		else if(running)
		{
			gameObject.transform.position = Vector3.MoveTowards(gameObject.transform.position, new Vector3(choosenPath[nextPathNodeIndex].x, choosenPath[nextPathNodeIndex].y, gameObject.transform.position.z), 2 * Time.fixedDeltaTime);
			if(isWife == CameraMovement.wifeSelected)
			{
				Camera.main.transform.position = new Vector3(gameObject.transform.position.x, Camera.main.transform.position.y, Camera.main.transform.position.z);
			}
		}
	}

	public bool IsRunning(){
		return running;
	}

	public float GetDuration(){
		return duration;
	}

	public void SetDuration(float pDuration){
		duration = pDuration;
	}

	public float GetElapsedTime(){
		return elapsedTime;
	}
	
	public void SetElapsedTime(float pElapsedTime){
		elapsedTime = pElapsedTime;
	}

	public void CopyProperties(Activity pActivity){
		path = pActivity.path;
		otherPath = pActivity.otherPath;
		duration = pActivity.duration;
		title = pActivity.title;
		description = pActivity.description;
		nextPathNodeIndex = 0;
		currentRoom = pActivity.currentRoom;
		elapsedTime = 0;
	}
	public List<Vector2> choosenPath;
	public List<Vector2> path; // currentRoom == playerLocation
	public List<Vector2> otherPath; // currentRoom != playerLocation
	public int nextPathNodeIndex;
	public int destinationPathNodeIndex;
	public float duration;
	public string title;
	public string description;

	public float elapsedTime;
	public bool running;
	public bool startedRunning;
	public Vector2 currentDirection;
	public ActivityType activityType;
	public Character.CurrentRoom currentRoom;
	public bool changingRoom;
	public bool startingPositionAhead;

	public GameObject mOwner;
	public bool isWife;
}
