﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using AnimPlugin;
using System;

public class Quest : MonoBehaviour
{

		public const int POWER = -4;
		public GameObject bedObj;
		public GameObject cleanerObj;
		public GameObject bathObj;
		public GameObject pramObj;
		public GameObject teddyObj;
		public GameObject boxObj;

		#region Consts

		public const string FACTOR_TIRE = "TIRE";
		public const string FACTOR_RAGE = "RAGE";
		public const float FactorModRotation = 1f;

		#endregion

		#region Fields - define in editor

		public string Name = "Define Name";
		public QuestExecuter PossibleExecuter = QuestExecuter.ANY;
		public bool CanRepeat = false;
		public int AddSubCash = 0;
		public float Duration = 0;
		public string ActivityName;
		public bool Done = false;
		public bool Todo = false;
		public bool ProduceHearth = false;

		public int Timeout;

		[SerializeField, HideInInspector]
		private int _MALE_RAGE = 0;
		[SerializeField, HideInInspector]
		private int _MALE_TIRE = 0;
		[SerializeField, HideInInspector]
		private int _FEM_RAGE = 0;
		[SerializeField, HideInInspector]
		private int _FEM_TIRE = 0;
		[SerializeField, HideInInspector]
		private int _EXE_RAGE = 0;
		[SerializeField, HideInInspector]
		private int _EXE_TIRE = 0;
		[SerializeField, HideInInspector]
		private int _OTH_RAGE = 0;
		[SerializeField, HideInInspector]
		private int _OTH_TIRE = 0;

		[SerializeField, HideInInspector]
		private string _InfoText = string.Empty;

		private QuestExecuter _Executing = QuestExecuter.BOTH;

		private Character Male;
		private Character Female;

		#endregion

		#region Properties

		public QuestExecuter Executing {
				set {
						this._Executing = value;
				}
				get { return this._Executing; }
		}

		public int PropHisRage {
				get { return this._MALE_RAGE; }
				set { this._MALE_RAGE = value; }
		}

		public int PropHisTire {
				get { return this._MALE_TIRE; }
				set { this._MALE_TIRE = value; }
		}

		public int PropHerRage {
				get { return this._FEM_RAGE; }
				set { this._FEM_RAGE = value; }
		}

		public int PropHerTire {
				get { return this._FEM_TIRE; }
				set { this._FEM_TIRE = value; }
		}

		public int PropExecRage {
				get { return this._EXE_RAGE; }
				set { this._EXE_RAGE = value; }
		}

		public int PropExecTire {
				get { return this._EXE_TIRE; }
				set { this._EXE_TIRE = value; }
		}

		public int PropOtherRage {
				get { return this._OTH_RAGE; }
				set { this._OTH_RAGE = value; }
		}

		public int PropOtherTire {
				get { return this._OTH_TIRE; }
				set { this._OTH_TIRE = value; }
		}

		public string InfoText {
				get { return this._InfoText; }
				set { this._InfoText = value; }
		}

		#endregion

		#region Logic/Coroutine methods

		void Awake ()
		{
				bedObj = GameObject.Find ("bed");
				cleanerObj = GameObject.Find ("cleaner");
				bathObj = GameObject.Find ("bath");
				pramObj = GameObject.Find ("pram");
				teddyObj = GameObject.Find ("teddy");
				boxObj = GameObject.Find ("blue_box");
		}

		void Start ()
		{
				bedObj.SetActive (false);
				cleanerObj.SetActive (false);
				bathObj.SetActive (false);
				pramObj.SetActive (false);
				teddyObj.SetActive (false);
				boxObj.SetActive (true);

				DayTimer.DayFinished += this.OnDayFinished;
		}

		
		public void ExecuteQuest (QuestExecuter executor)
		{
				if (this.Timeout > 0 && this.Name != "Rest") {
						Toast.T ("Already done that today");
						return;
				}	
				if (this.Todo && this.Done) {
						Toast.T ("What was done cannot be undone ;)");
						return;
				}
				GameObject g = GameObject.Find ("Hearth");
				// this in todo list
				if (this.Todo) {
						
						if (g == null) {

								Toast.T ("This Requires some heart...");
								return;
						}
				}
				AdditionalPossible ap = this.gameObject.GetComponent<AdditionalPossible> ();
				if (ap != null) {
					
						if (!ap.IsPossible (this)) {
								return;
						}
				}
				if (!IsPossible ()) {
						Toast.T ("Not enough cash!");
						return;
				}
				this.Male = GameObject.Find ("husband").GetComponent<Character> ();
				this.Female = GameObject.Find ("wife").GetComponent<Character> ();
				this._Executing = executor;
				if (/*!Master.Instance.IsAvailable (executor)*/ false) {
               
				} else {
						Debug.Log (" type is set to " + Executing);
						if (!String.IsNullOrEmpty (this.ActivityName)) {
								List<Activity> l = GameObject.Find ("TmpActivities").GetComponent<ActivitiesContainer> ().activities;
								GameObject person = null;
								try {
										for (int i = 0; i < l.Count; i++) {
												if (l [i].title.Equals (this.ActivityName)) {
														if (Executing == QuestExecuter.MALE) {
																person = GameObject.Find ("husband");
														} else if (Executing == QuestExecuter.FEMALE)
																person = GameObject.Find ("wife");
												
														if (person != null) {
																Activity a = person.AddComponent<Activity> ();
																a.CopyProperties (l [i]);
																a.duration = this.Duration * 120;
																a.Run ();
														} else if (this.Executing == QuestExecuter.BOTH) {
																Activity a = GameObject.Find ("husband").AddComponent<Activity> ();
																a.CopyProperties (l [i]);
																a.duration = this.Duration * 120;
																a.Run ();   
																a = GameObject.Find ("wife").AddComponent<Activity> ();
																a.CopyProperties (l [i]);
																a.duration = this.Duration * 120;
																a.Run ();   
														}
												}	
										}
								} catch (Exception e) {
										Debug.Log (e.Message);
								}

						} else if (this.GetComponent<FastActivity> () != null) {
								GameObject person = null;
								if (Executing == QuestExecuter.MALE) {
										person = GameObject.Find ("husband");
								} else if (Executing == QuestExecuter.FEMALE) {
										person = GameObject.Find ("wife");
								}
								if (person != null) {
										FastActivity ff = person.AddComponent<FastActivity> ();
										ff.GoThere = this.GetComponent<FastActivity> ().GoThere;
										ff.WaitFor = this.Duration;
										ff.Init ();
								} else if (this.Executing == QuestExecuter.BOTH) {
										person = GameObject.Find ("husband");
										FastActivity ff = person.AddComponent<FastActivity> ();
										ff.GoThere = this.GetComponent<FastActivity> ().GoThere;
										ff.WaitFor = this.Duration;
										ff.Init ();
										person = GameObject.Find ("wife");
										ff = person.AddComponent<FastActivity> ();
										ff.GoThere = this.GetComponent<FastActivity> ().GoThere;
										ff.WaitFor = this.Duration;
										ff.Init ();
								}
						}
						if (executor == QuestExecuter.BOTH) {
								// some special other logic here
								this.ExecuteByBoth ();
						} else {
								Male.decreaseSatisfaction (_MALE_RAGE * POWER);
								Debug.Log ("inc his rage by " + _MALE_RAGE * POWER);
								Male.decreaseRelax (this._MALE_TIRE * POWER);
								Debug.Log (this._MALE_TIRE * POWER);
								Female.decreaseSatisfaction (_FEM_RAGE * POWER);
								Debug.Log (this._FEM_RAGE * POWER);
								Female.decreaseRelax (_FEM_TIRE * POWER);
								Debug.Log (this._FEM_TIRE * POWER);
								if (this._Executing == QuestExecuter.MALE) {
										//His. Rage +=

										Male.decreaseSatisfaction (_EXE_RAGE * POWER);
										//Debug.Log(this._EXE_RAGE);
										Female.decreaseSatisfaction (_OTH_RAGE * POWER);
										//Debug.Log(this._OTH_RAGE);
										Male.decreaseRelax (_EXE_TIRE * POWER);
										//Debug.Log(this._EXE_TIRE);
										Female.decreaseRelax (_OTH_TIRE * POWER);
										//Debug.Log(this._OTH_TIRE);
								} else {
										//His. Rage +=
										Female.decreaseSatisfaction (_EXE_RAGE * POWER);
										//Debug.Log(this._EXE_RAGE);
										Male.decreaseSatisfaction (_OTH_RAGE * POWER);
										//Debug.Log(this._OTH_RAGE);
										Female.decreaseRelax (_EXE_TIRE * POWER);
										//Debug.Log(this._EXE_TIRE);
										Male.decreaseRelax (_OTH_TIRE * POWER);
										//Debug.Log(this._OTH_TIRE);
								}
			
								switch (this.Name) {
								case "Buy a Crib":
										bedObj.SetActive (true);
										break;
								case "Buy Vacuum Cleaner":
										cleanerObj.SetActive (true);
										break;
								case "Buy baby batchtoob.":
										bathObj.SetActive (true);
										break;
								case "Buy a Pram":
										pramObj.SetActive (true);
										break;
								case "Buy baby toys":
										teddyObj.SetActive (true);
										break;
								case "Sell PS":
										boxObj.SetActive (false);
										break;
								
								}
						}
						Cash.Own += AddSubCash;
						// lock executor
						// invoke release executor after time
						if (Todo && g != null) {
								GameObject.Destroy (g);
								this.Done = true;
						}
						if (this.ProduceHearth) {
								GameObject exec = null;
								if (executor == QuestExecuter.MALE) {
										exec = GameObject.Find ("wife");

								} else {
										exec = GameObject.Find ("husband");
								}
								GameObject.Instantiate (Camera.main.GetComponent<Res> ().Hearth, exec.transform.position, Quaternion.identity);
						}
						Timeout++;
				}
		}

		public virtual void ExecuteByBoth ()
		{
				//Some special logic here?
		}

		public void OnDayFinished ()
		{
				if (Timeout > 0)
						Timeout--;	
		}

		public bool IsPossible ()
		{
				return Cash.Own + this.AddSubCash > 0;
		}

		#endregion
}

#region Helper classes
[System.Serializable]
public enum QuestExecuter
{
		MALE,
		FEMALE,
		ANY,
		// ANY also count as self
		OTHER,
		BOTH
		// BOTH is special!
}
#endregion